#include <Arduino.h>
#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEServer.h>
#include <Adafruit_NeoPixel.h>

#define PIN_NEOPIXEL    10    // Pin al que está conectado el Neopixel (D2)
#define NUM_PIXELS      1    // Solo un Neopixel conectado

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_PIXELS, PIN_NEOPIXEL, NEO_GRB + NEO_KHZ800);

BLECharacteristic *pCharacteristic;
bool deviceConnected = false;

class MyServerCallbacks : public BLEServerCallbacks {
    void onConnect(BLEServer* pServer) {
        deviceConnected = true;
    }

    void onDisconnect(BLEServer* pServer) {
        deviceConnected = false;
    }
};

void setup() {
    Serial.begin(115200);

    // Configuración del Neopixel
    strip.begin();
    strip.show();  // Inicializar el Neopixel en apagado

    // Configuración del Bluetooth
    BLEDevice::init("NeopixelControl");
    BLEServer *pServer = BLEDevice::createServer();
    pServer->setCallbacks(new MyServerCallbacks());
    BLEService *pService = pServer->createService(BLEUUID("19b10000-e8f2-537e-4f6c-d104768a1214")); // UUID del servicio
    pCharacteristic = pService->createCharacteristic(
                          BLEUUID("19b10001-e8f2-537e-4f6c-d104768a1214"),
                          BLECharacteristic::PROPERTY_WRITE
                        ); // UUID de la característica de escritura (color)
    pService->start();
    BLEAdvertising *pAdvertising = pServer->getAdvertising();
    pAdvertising->addServiceUUID(pService->getUUID());
    pAdvertising->start();

    Serial.println("Dispositivo Bluetooth activo, esperando conexión...");
}

void loop() {
    if (deviceConnected) {
        // Revisar si la característica tiene un valor nuevo para leer
        if (pCharacteristic->getValue().length() > 0) {
            // Obtener el valor como String
            String value = pCharacteristic->getValue().c_str();

            // Convertir el String hexadecimal a un valor entero (color)
            uint32_t color = (uint32_t) strtol(value.c_str(), NULL, 16);

            // Actualizar el Neopixel con el color recibido
            strip.setPixelColor(0, color);
            strip.show();

            // Limpiar el valor de la característica después de procesarlo
            pCharacteristic->setValue("");
        }
    }
    delay(10);
}
